@@ -38,8 +38,9 @@
 #include <libxfce4ui/libxfce4ui.h>
 
 #include <gdk/gdk.h>
-#ifdef GDK_WINDOWING_X11
+#ifdef ENABLE_X11
 #include <X11/Xlib.h>
+#include <X11/Xatom.h>
 #include <X11/Xutil.h>
 #endif
 
@@ -56,6 +57,7 @@
 
 
 static void     terminal_app_finalize                 (GObject            *object);
+static void     terminal_app_update_accels            (TerminalApp        *app);
 static void     terminal_app_update_windows_accels    (gpointer            user_data);
 static gboolean terminal_app_accel_map_load           (gpointer            user_data);
 static gboolean terminal_app_accel_map_save           (gpointer            user_data);
@@ -129,12 +131,16 @@ static void
 terminal_app_init (TerminalApp *app)
 {
   app->preferences = terminal_preferences_get ();
+  g_signal_connect_swapped (G_OBJECT (app->preferences), "notify::shortcuts-no-menukey",
+                            G_CALLBACK (terminal_app_update_accels), app);
 
   /* remember the original menu bar accel */
   g_object_get (G_OBJECT (gtk_settings_get_default ()),
                 "gtk-menu-bar-accel", &app->initial_menu_bar_accel,
                 NULL);
 
+  terminal_app_update_accels (app);
+
   /* schedule accel map load and update windows when finished */
   app->accel_map_load_id = gdk_threads_add_idle_full (G_PRIORITY_LOW, terminal_app_accel_map_load, app,
                                                       terminal_app_update_windows_accels);
@@ -169,6 +175,7 @@ terminal_app_finalize (GObject *object)
     }
   g_slist_free (app->windows);
 
+  g_signal_handlers_disconnect_by_func (G_OBJECT (app->preferences), G_CALLBACK (terminal_app_update_accels), app);
   g_object_unref (G_OBJECT (app->preferences));
 
   if (app->initial_menu_bar_accel != NULL)
@@ -182,6 +189,22 @@ terminal_app_finalize (GObject *object)
 
 
 
+static void
+terminal_app_update_accels (TerminalApp *app)
+{
+  gboolean no_key;
+
+  g_object_get (G_OBJECT (app->preferences),
+                "shortcuts-no-menukey", &no_key,
+                NULL);
+  g_object_set (G_OBJECT (gtk_settings_get_default ()),
+                "gtk-menu-bar-accel",
+                no_key ? NULL : app->initial_menu_bar_accel,
+                NULL);
+}
+
+
+
 static void
 terminal_app_update_windows_accels (gpointer user_data)
 {
@@ -407,6 +430,207 @@ terminal_app_new_window (TerminalWindow *window,
 
 
 
+/*
+  Derived from XParseGeometry() in XFree86  
+
+  Copyright 1985, 1986, 1987,1998  The Open Group
+
+  All Rights Reserved.
+
+  The above copyright notice and this permission notice shall be included
+  in all copies or substantial portions of the Software.
+
+  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+  IN NO EVENT SHALL THE OPEN GROUP BE LIABLE FOR ANY CLAIM, DAMAGES OR
+  OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+  OTHER DEALINGS IN THE SOFTWARE.
+
+  Except as contained in this notice, the name of The Open Group shall
+  not be used in advertising or otherwise to promote the sale, use or
+  other dealings in this Software without prior written authorization
+  from The Open Group.
+*/
+
+
+/*
+ *    XParseGeometry parses strings of the form
+ *   "=<width>x<height>{+-}<xoffset>{+-}<yoffset>", where
+ *   width, height, xoffset, and yoffset are unsigned integers.
+ *   Example:  “=80x24+300-49”
+ *   The equal sign is optional.
+ *   It returns a bitmask that indicates which of the four values
+ *   were actually found in the string.  For each value found,
+ *   the corresponding argument is updated;  for each value
+ *   not found, the corresponding argument is left unchanged. 
+ */
+
+/* The following code is from Xlib, and is minimally modified, so we
+ * can track any upstream changes if required.  Don’t change this
+ * code. Or if you do, put in a huge comment marking which thing
+ * changed.
+ */
+
+static int
+read_int (gchar   *string,
+          gchar  **next)
+{
+  int result = 0;
+  int sign = 1;
+  
+  if (*string == '+')
+    string++;
+  else if (*string == '-')
+    {
+      string++;
+      sign = -1;
+    }
+
+  for (; (*string >= '0') && (*string <= '9'); string++)
+    {
+      result = (result * 10) + (*string - '0');
+    }
+
+  *next = string;
+
+  if (sign >= 0)
+    return (result);
+  else
+    return (-result);
+}
+
+/* 
+ * Bitmask returned by XParseGeometry().  Each bit tells if the corresponding
+ * value (x, y, width, height) was found in the parsed string.
+ */
+#ifndef ENABLE_X11
+#define NoValue         0x0000
+#define XValue          0x0001
+#define YValue          0x0002
+#define WidthValue      0x0004
+#define HeightValue     0x0008
+#define AllValues       0x000F
+#define XNegative       0x0010
+#define YNegative       0x0020
+#endif
+
+/* Try not to reformat/modify, so we can compare/sync with X sources */
+static int
+gtk_XParseGeometry (const char   *string,
+                    int          *x,
+                    int          *y,
+                    unsigned int *width,   
+                    unsigned int *height)  
+{
+  int mask = NoValue;
+  char *strind;
+  unsigned int tempWidth, tempHeight;
+  int tempX, tempY;
+  char *nextCharacter;
+
+  /* These initializations are just to silence gcc */
+  tempWidth = 0;
+  tempHeight = 0;
+  tempX = 0;
+  tempY = 0;
+  
+  if ( (string == NULL) || (*string == '\0')) return(mask);
+  if (*string == '=')
+    string++;  /* ignore possible '=' at beg of geometry spec */
+
+  strind = (char *)string;
+  if (*strind != '+' && *strind != '-' && *strind != 'x') {
+    tempWidth = read_int(strind, &nextCharacter);
+    if (strind == nextCharacter) 
+      return (0);
+    strind = nextCharacter;
+    mask |= WidthValue;
+  }
+
+  if (*strind == 'x' || *strind == 'X') {	
+    strind++;
+    tempHeight = read_int(strind, &nextCharacter);
+    if (strind == nextCharacter)
+      return (0);
+    strind = nextCharacter;
+    mask |= HeightValue;
+  }
+
+  if ((*strind == '+') || (*strind == '-')) {
+    if (*strind == '-') {
+      strind++;
+      tempX = -read_int(strind, &nextCharacter);
+      if (strind == nextCharacter)
+        return (0);
+      strind = nextCharacter;
+      mask |= XNegative;
+
+    }
+    else
+      {	strind++;
+      tempX = read_int(strind, &nextCharacter);
+      if (strind == nextCharacter)
+        return(0);
+      strind = nextCharacter;
+      }
+    mask |= XValue;
+    if ((*strind == '+') || (*strind == '-')) {
+      if (*strind == '-') {
+        strind++;
+        tempY = -read_int(strind, &nextCharacter);
+        if (strind == nextCharacter)
+          return(0);
+        strind = nextCharacter;
+        mask |= YNegative;
+
+      }
+      else
+        {
+          strind++;
+          tempY = read_int(strind, &nextCharacter);
+          if (strind == nextCharacter)
+            return(0);
+          strind = nextCharacter;
+        }
+      mask |= YValue;
+    }
+  }
+	
+  /* If strind isn't at the end of the string the it's an invalid
+		geometry specification. */
+
+  if (*strind != '\0') return (0);
+
+  if (mask & XValue)
+    *x = tempX;
+  if (mask & YValue)
+    *y = tempY;
+  if (mask & WidthValue)
+    *width = tempWidth;
+  if (mask & HeightValue)
+    *height = tempHeight;
+  return (mask);
+}
+
+static int
+parse_geometry (const char   *string,
+                int          *x,
+                int          *y,
+                unsigned int *width,
+                unsigned int *height)
+{
+#ifdef ENABLE_X11
+  if (WINDOWING_IS_X11 ())
+    return XParseGeometry (string, x, y, width, height);
+#endif
+  /* stolen from GTK 3.24.38 */
+  return gtk_XParseGeometry (string, x, y, width, height);
+}
+
+
+
 static void
 terminal_app_new_window_with_terminal (TerminalWindow *existing,
                                        TerminalScreen *terminal,
@@ -448,17 +672,15 @@ terminal_app_new_window_with_terminal (TerminalWindow *existing,
   if (G_UNLIKELY (terminal_window_is_drop_down (existing)))
     {
       /* resize new window to the default geometry */
-#ifdef GDK_WINDOWING_X11
       gchar *geo;
       guint  w, h;
       g_object_get (G_OBJECT (app->preferences), "misc-default-geometry", &geo, NULL);
       if (G_LIKELY (geo != NULL))
         {
-          XParseGeometry (geo, NULL, NULL, &w, &h);
+          parse_geometry (geo, NULL, NULL, &w, &h);
           g_free (geo);
           terminal_screen_force_resize_window (terminal, GTK_WINDOW (window), w, h);
         }
-#endif
     }
   else
     {
@@ -635,6 +857,68 @@ terminal_app_find_screen_by_name (const gchar *display_name)
 
 
 
+static void
+move_window_to_saved_workspace (GtkWidget *window,
+                                gpointer   user_data)
+{
+#ifdef ENABLE_X11
+  GdkDisplay *gdk_display;
+  GdkScreen  *gdk_screen;
+  Display    *display;
+  Window      xwin;
+  Window      rootwin;
+  Atom        message;
+  XEvent      event;
+#endif
+
+  g_signal_handlers_disconnect_by_func (window,
+                                        move_window_to_saved_workspace,
+                                        user_data);
+
+#ifdef ENABLE_X11
+  if (WINDOWING_IS_X11 ())
+    {
+      // EWMH says that we can set _NET_WM_DESKTOP on our own window before
+      // mapping it, and the WM should honor our request.  However, xfwm4
+      // doesn't appear to do that (even though an inspection of the code seems
+      // to suggest it does).  What seems more reliable (and I believe what
+      // Firefox does) is to wait until the window is mapped, and then send a
+      // ClientMessage to the root window to ask the WM to move us.
+
+      gdk_display = gtk_widget_get_display (GTK_WIDGET (window));
+      gdk_screen = gtk_widget_get_screen (GTK_WIDGET (window));
+
+      gdk_x11_display_error_trap_push (gdk_display);
+
+      display = gdk_x11_display_get_xdisplay (gdk_display);
+      rootwin = gdk_x11_window_get_xid (gdk_screen_get_root_window (gdk_screen));
+      xwin = gdk_x11_window_get_xid (gtk_widget_get_window (GTK_WIDGET (window)));
+      message = XInternAtom (display, "_NET_WM_DESKTOP", False);
+
+      event.type = ClientMessage;
+      event.xclient.serial = 0;
+      event.xclient.send_event = True;
+      event.xclient.display = display;
+      event.xclient.window = xwin;
+      event.xclient.message_type = message;
+      event.xclient.format = 32;
+      event.xclient.data.l[0] = GPOINTER_TO_INT (user_data);
+      event.xclient.data.l[1] = 1;  // Source type: application
+      event.xclient.data.l[2] = 0;
+      event.xclient.data.l[3] = 0;
+      event.xclient.data.l[4] = 0;
+
+      XSendEvent (display, rootwin, False,
+                  SubstructureNotifyMask | SubstructureRedirectMask,
+                  &event);
+
+      gdk_x11_display_error_trap_pop_ignored (gdk_display);
+    }
+#endif
+}
+
+
+
 static void
 terminal_app_open_window (TerminalApp        *app,
                           TerminalWindowAttr *attr)
@@ -649,13 +933,12 @@ terminal_app_open_window (TerminalApp        *app,
   GdkDisplay      *attr_display;
   gint             attr_screen_num;
   gint             active_tab = -1, i, existing_tabs = 0;
-#ifdef GDK_WINDOWING_X11
+  guint            width = 80, height = 24;
   GdkGravity       gravity = GDK_GRAVITY_NORTH_WEST;
   gint             mask = NoValue, x, y, new_x, new_y;
-  guint            width = 1, height = 1, new_width, new_height;
+  guint            new_width, new_height;
   gint             screen_width = 0, screen_height = 0;
   gint             window_width, window_height;
-#endif
 
   terminal_return_if_fail (TERMINAL_IS_APP (app));
   terminal_return_if_fail (attr != NULL);
@@ -761,6 +1044,14 @@ terminal_app_open_window (TerminalApp        *app,
           gtk_window_set_screen (GTK_WINDOW (window), screen);
           g_object_unref (G_OBJECT (screen));
         }
+
+      if (attr->workspace >= 0)
+        {
+          g_signal_connect_after (window,
+                                  "map",
+                                  G_CALLBACK (move_window_to_saved_workspace),
+                                  GINT_TO_POINTER (attr->workspace));
+        }
     }
 
   terminal_window_set_scrollbar_visibility (TERMINAL_WINDOW (window), attr->scrollbar);
@@ -777,16 +1068,16 @@ terminal_app_open_window (TerminalApp        *app,
     {
       /* try to apply the geometry to the window */
       g_object_get (G_OBJECT (app->preferences), "misc-default-geometry", &geometry, NULL);
-#ifdef GDK_WINDOWING_X11
+
       /* defaults */
-      mask = XParseGeometry (geometry, &x, &y, &width, &height);
+      mask = parse_geometry (geometry, &x, &y, &width, &height);
 
       /* geometry provided via command line parameter */
       if (G_UNLIKELY (attr->geometry != NULL))
         {
           g_free (geometry);
           geometry = g_strdup (attr->geometry);
-          mask = XParseGeometry (geometry, &new_x, &new_y, &new_width, &new_height);
+          mask = parse_geometry (geometry, &new_x, &new_y, &new_width, &new_height);
 
           if (mask & WidthValue)
             width = new_width;
@@ -797,7 +1088,6 @@ terminal_app_open_window (TerminalApp        *app,
           if (mask & YValue)
             y = new_y;
         }
-#endif
     }
 
   /* special handling for --active-tab in case we are adding tabs to an existing window */
@@ -824,7 +1114,6 @@ terminal_app_open_window (TerminalApp        *app,
   if (!attr->drop_down)
     {
       /* move the window to desired position */
-#ifdef GDK_WINDOWING_X11
       if ((mask & XValue) || (mask & YValue))
         {
           screen = gtk_window_get_screen (GTK_WINDOW (window));
@@ -845,9 +1134,6 @@ terminal_app_open_window (TerminalApp        *app,
           gtk_window_move (GTK_WINDOW (window), x, y);
         }
       else if (!(mask & WidthValue) && !(mask & XValue))
-#else
-      if (!gtk_window_parse_geometry (GTK_WINDOW (window), geometry))
-#endif
         g_warning ("Invalid geometry string \"%s\"", geometry);
 
       /* cleanup */
