@@ -28,10 +28,12 @@
 
 #include <libxfce4util/libxfce4util.h>
 
-#include <gdk/gdk.h>
-#ifdef GDK_WINDOWING_X11
+#ifdef ENABLE_X11
 #include <X11/Xlib.h>
 #endif
+#ifdef HAVE_GTK_LAYER_SHELL
+#include <gtk-layer-shell/gtk-layer-shell.h>
+#endif
 
 #include <terminal/terminal-util.h>
 #include <terminal/terminal-private.h>
@@ -92,54 +94,73 @@ terminal_util_show_about_dialog (GtkWindow *parent)
 
 
 
+#ifdef HAVE_GTK_LAYER_SHELL
+static void
+is_active_changed (GtkWindow *window)
+{
+  g_signal_handlers_disconnect_by_func (window, is_active_changed, NULL);
+  gtk_layer_set_keyboard_mode (window, GTK_LAYER_SHELL_KEYBOARD_MODE_ON_DEMAND);
+}
+#endif
+
+
+
 void
 terminal_util_activate_window (GtkWindow *window)
 {
-#ifdef GDK_WINDOWING_X11
-  guint32              timestamp;
-  XClientMessageEvent  event;
-  GdkWindow           *gdk_window = gtk_widget_get_window (GTK_WIDGET (window));
-  GdkDisplay          *display = gdk_window_get_display (gdk_window);
-
   terminal_return_if_fail (GTK_IS_WINDOW (window));
   terminal_return_if_fail (gtk_widget_get_realized (GTK_WIDGET (window)));
 
-  if (!GDK_IS_X11_WINDOW (gdk_window))
-    return;
-
   /* leave if the window is already active */
   if (gtk_window_is_active (window))
     return;
 
-  timestamp = gtk_get_current_event_time ();
-  if (timestamp == 0)
-    timestamp = gdk_x11_get_server_time (gdk_window);
+#ifdef ENABLE_X11
+  if (WINDOWING_IS_X11 ())
+    {
+      guint32              timestamp;
+      XClientMessageEvent  event;
+      GdkWindow           *gdk_window = gtk_widget_get_window (GTK_WIDGET (window));
+      GdkDisplay          *display = gdk_window_get_display (gdk_window);
 
-  /* we need a slightly custom version of the call through Gtk+ to
-   * properly focus the panel when a plugin calls
-   * xfce_panel_plugin_focus_widget() */
-  event.type = ClientMessage;
-  event.window = GDK_WINDOW_XID (gdk_window);
-  event.message_type = gdk_x11_get_xatom_by_name ("_NET_ACTIVE_WINDOW");
-  event.format = 32;
-  event.data.l[0] = 1; /* app */
-  event.data.l[1] = timestamp;
-  event.data.l[2] = event.data.l[3] = event.data.l[4] = 0;
+      timestamp = gtk_get_current_event_time ();
+      if (timestamp == 0)
+        timestamp = gdk_x11_get_server_time (gdk_window);
 
-  gdk_x11_display_error_trap_push (display);
+      /* we need a slightly custom version of the call through Gtk+ to
+       * properly focus the panel when a plugin calls
+       * xfce_panel_plugin_focus_widget() */
+      event.type = ClientMessage;
+      event.window = GDK_WINDOW_XID (gdk_window);
+      event.message_type = gdk_x11_get_xatom_by_name ("_NET_ACTIVE_WINDOW");
+      event.format = 32;
+      event.data.l[0] = 1; /* app */
+      event.data.l[1] = timestamp;
+      event.data.l[2] = event.data.l[3] = event.data.l[4] = 0;
 
-  XSendEvent (gdk_x11_get_default_xdisplay (),
-              gdk_x11_get_default_root_xwindow (), False,
-              StructureNotifyMask, (XEvent *) &event);
+      gdk_x11_display_error_trap_push (display);
 
-  gdk_display_flush (display);
+      XSendEvent (gdk_x11_get_default_xdisplay (),
+                  gdk_x11_get_default_root_xwindow (), False,
+                  StructureNotifyMask, (XEvent *) &event);
 
-  if (gdk_x11_display_error_trap_pop (display) != 0)
-    g_critical ("Failed to focus window");
-#else
-  /* our best guess on non-x11 clients */
-  gtk_window_present (window);
+      gdk_display_flush (display);
+
+      if (gdk_x11_display_error_trap_pop (display) != 0)
+        g_critical ("Failed to focus window");
+    }
+  else
 #endif
+#ifdef HAVE_GTK_LAYER_SHELL
+  if (gtk_layer_is_supported () && gtk_layer_is_layer_window (window))
+    {
+      gtk_layer_set_keyboard_mode (window, GTK_LAYER_SHELL_KEYBOARD_MODE_EXCLUSIVE);
+      g_signal_connect (window, "notify::is-active", G_CALLBACK (is_active_changed), NULL);
+    }
+  else
+#endif
+    /* our best guess on non-x11 and non layer-shell clients */
+    gtk_window_present (window);
 }
 
 
